
#package Commands;
# We can't explicitly say it's the package Commands because
# we want to keep the main namespace. :/

=pod 

This is were all \commands are defined. If you want to add
your own \command, create your function and add an entry to
the %::commands hash. For example:

	$::commands{ stuff } = \&Commands::stuff;

	sub stuff{ ymusk_message( "Command 'stuff' invoked );

The subroutine receives whatever follow the command in a 
single string. 

	sub stuff
	{
		my $arg = shift;
		ymusk_message( "command stuff caleld with arguments '$arg'" );
	}

=cut

$commands{eval} = sub{ my $o = eval shift; ymusk_message( $o ) if $o;
                       ymusk_message( $@ ) if $@ };
$commands{ quit } = sub{ exit };
$commands{ help } = \&Commands::help;

sub Commands::help
{
	ymusk_message( join ' ', sort( keys %commands ) );	
}				  

$commands{ hook } = \&Commands::hook;
sub Commands::hook
{
	$_[0] =~ /(\S.*?)\s*=\s*(.*)/; 
	ymusk_message( "Bad hook command: $_[0]" ) and return unless $1 and $2;
	$hooks{ eval $1 } = eval "sub{ $2 ; return () }";
}

$commands{ bind } = \&Commands::bind;

sub Commands::bind
{
	$_[0] =~ /(<.*?>)\s*=\s*(.*)/; 
	ymusk_message( "Bad bind command: $_[0]" ) and return unless $1 and $2;
	$input->bind( $1, eval "sub{ $2 }" );
}

$commands{ do } = \&Commands::do;

sub Commands::do 
{
	open FILE, shift or ymusk_message( $! ) and return;
	foreach( <FILE> )
	{
		$input->insert( 'end', $_ );
		parse_input();
	}
	ymusk_message( "File parsed" );

	close FILE;
}


$commands{ connect } = \&Commands::connect;

sub Commands::connect
{
   my @params = split ' ', shift;
   
   unless( ( @params == 2 ) or ( @params == 4 ) )
   {
     ymusk_message( '/connect host port [ character password ]' );
	 return;
   }

   my ( $host, $port, $password );
   ( $host, $port, $character, $password ) = @params;

	# close previous connection (if any)
   $telnet->close();
   $telnet->host( $host );
   $telnet->port( $port );

   $telnet->errmode( \&ymusk_message ); # just for the open()
   $telnet->errmsg();
   $telnet->open();
   $telnet->errmode( 'return' );

	return if $telnet->errmsg();

   if( $character ){ &send( "connect $character $password\n" ); }
   
   start_log( $character || $host );

   # change the main window title
   $mw->configure( -title => "Ymusk - " . ( $character || $host ) );

   $time_online = time;
}

$commands{ disconnect } = \&Commands::disconnect;

sub Commands::disconnect
{ 
	$telnet->close(); 
	ymusk_message( "connection with " . $telnet->host() ." closed" );
	$mw->configure( -title => 'Ymusk' );
	$time_online = 0;
	$character = '';
}	

$commands{ margin } = \&Commands::margin;

sub Commands::margin
{
	my $size = shift;
	return unless $size =~ /[+-]?\d+/;

	if( $size =~ /[+-]/ )
	{
		$size += $output->tagCget( 'margin' => '-lmargin2' );
		$size = 0 if $size < 0
	}

	$output->tagConfigure( 'margin', -lmargin2 => $size );
	$output->tagAdd( 'margin', '1.0', 'end' );
}

$commands{ paragraph } = \&Commands::paragraph;
sub Commands::paragraph
{
    my $size = shift;
    return unless $size =~ /[+-]?\d+/;

    if( $size =~ /[+-]/ )
    {
        $size += $output->cget( '-spacing3' );
        $size = 0 if $size < 0
    }

    $output->configure( -spacing3 => $size );
}

$commands{ tag } = \&Commands::tag;
sub Commands::tag
{
	$_[0] =~ /(\S.*?)\s*=\s*(.*\S)/; 
	ymusk_message( "Bad bind command: $_[0]" ) and return unless $1 and $2;
	$output->tagConfigure( $1, eval( $2 ) );
	
}

$commands{ log } = \&Commands::log;

sub Commands::log
{
	my @params = split ' ', shift;
	
	if( @params == 0 )
	{
		if( $logfile )
		{
			ymusk_message( "the log file is $logfile" );
		}
		else
		{
			ymusk_message( "log not enabled" )
		}
	}
	elsif( @params > 1 )
	{
		ymusk_message( "/log [ off|<filename>]\n" )	
	}
	elsif( $params[0] =~ /^off$/i )
	{
		close LOG;
		$logfile = ''; 
		$output->insert( 'end', "YMUSK: log file closed\n" );
	}
	else
	{
		$logfile = $params[0];
		if( open LOG, ">>$logfile" )
		{
			$output->insert( 'end', "YMUSK: logging at $logfile\n" );
		}
		else
		{
			$output->insert( 'end', "YMUSK: error in opening log file ($!)" );
		}
	}
}


'end of module Commands';
